<?php
/**
 * Questo file è parte del plugin WooCommerce v3.x di Fattura24
 * Autore: Fattura24.com <info@fattura24.com>
 *
 * metodi utilizzati da hooks.php correlati alla gestione
 * e all'aggiunta dei campi fiscali - lato utente, lato checkout, lato admin
 *
 */

namespace fattura24;

if (!defined('ABSPATH')) {
    exit;
}

function fatt_24_show_billing_checkbox()
{
    if ('2' == get_option('fatt-24-inv-create') && fatt_24_get_flag(FATT_24_INV_DISABLE_RECEIPTS)) {
        return false;
    }

    $showCheckbox = get_option('fatt-24-toggle-billing-fields') == '';
    $piRequired = fatt_24_get_flag(FATT_24_ABK_VATCODE_REQ) == 1;
    $result = true;

    if ($piRequired || !$showCheckbox) {
        $result = false;
    }

    return $result;
}

function fatt_24_delete_user($user_id)
{
    $f24UserMetaArray = ['billing_fiscalcode', 'billing_vatcode', 'billing_pecaddress', 'billing_recipientcode'];
    foreach ($f24UserMetaArray as $metakey) {
        delete_user_meta($user_id, $metakey);
    }
}

/**
 * Campi aggiunti al checkout
 */
function fatt_24_billing_checkout_fields($fields)
{
    $showCheckbox = fatt_24_show_billing_checkbox();

    if ($showCheckbox) {
        $fields['billing']['billing_checkbox'] = array(
            'type'     => 'checkbox',
            'label'    => __('I would like to receive an invoice', 'fattura24'),
            'class'    => array('form-row-wide', 'fatt_24_billing_cb'),
            'priority' => 25,
            'clear'    => true
        );
    }

    $fields['billing']['billing_fiscalcode'] = array(
            'type'       => 'text',
            'placeholder' => __('Codice fiscale', 'fattura24'),
            'label'      => __('Codice fiscale', 'fattura24'),
            'class'      => array('form-row-wide', 'inv_create_elecinvoice'), // lo aggiungo ai campi nascosti se il paese != IT
            'priority'   => 30,
            'clear'      => true
         );

    $fields['billing']['billing_vatcode'] = array(
            'type'       => 'text',
            'placeholder' => __('Partita Iva / VAT', 'fattura24'),
            'label'      => __('Partita Iva / VAT', 'fattura24'),
            'class'      => array('form-row-wide'),
            'priority'   => 30,
            'clear'      => true,
        );

    $fields['billing']['billing_recipientcode'] = array(
            'type' => 'text',
            'placeholder' => __('Codice Destinatario', 'fattura24'),
            'label' => __('Codice Destinatario', 'fattura24'),
            'maxlength' => 7, // definisco la lunghezza massima dell'input
            'class' => array('form-row-wide','inv_create_elecinvoice'),
            'priority' => 30, // cambio l'ordine dei campi nel layout
            'clear' => true
        );

    $fields['billing']['billing_pecaddress'] = array(
             'type' => 'email',
             'placeholder' => __('Indirizzo PEC', 'fattura24'),
             'label' => __('Indirizzo PEC', 'fattura24'),
             'validate' => array('email'),
             'class' => array('form-row-wide','inv_create_elecinvoice'),
             'priority' => 30,
             'clear' => true
        );

    return $fields;
}

/**
 * I campi sono gli stessi già settati sopra,
 * la checkbox deve essere visibile solo nel checkout
 */
function fatt_24_billing_fields($fields)
{
    $url = $_SERVER['REQUEST_URI'];
    $isCheckout = preg_match('/checkout/', $url) == 1;
    $f24BillingFields = fatt_24_billing_checkout_fields($fields)['billing'];
    $showCheckbox = fatt_24_show_billing_checkbox();

    if ($showCheckbox && !$isCheckout) {
        unset($f24BillingFields['billing_checkbox']);
    }

    foreach ($f24BillingFields as $key => $field) {
        $fields[$key] = $field;
    }

    return $fields;
}

function fatt_24_checkout_fields_validation($data, $errors)
{
    global $woocommerce;
    $cValid = true; // valore di default
    //La logica di convalida della p. iva ora è legata anche alla checkbox lato frontend
    $piRequired = fatt_24_get_flag(FATT_24_ABK_VATCODE_REQ) == 1;
    $cfRequired = fatt_24_CF_flag();
    $customer = $woocommerce->customer ? $woocommerce->customer : new \WC_Customer(0, true);
    $country = method_exists($customer, 'get_billing_country') ? $customer->get_billing_country() : $customer->get_country(); // deprecato da WooCommerce 3.0
    $feRequired = fatt_24_get_resulting_doc_type($country) == FATT_24_DT_FATTURA_ELETTRONICA || isset($_POST['billing_checkbox']) && (int) $_POST['billing_checkbox'] == 1;
    $pILen = strlen(trim($_POST['billing_vatcode'])) === 11; // lunghezza cambiata per intercettare p.iva straniere
    $cFisLen = strlen(trim($_POST['billing_fiscalcode']));
    $cValid = ($cFisLen == 11 || $cFisLen == 16) ? true : false;


    if ($feRequired && empty($_POST['billing_fiscalcode']) && (empty($_POST['billing_vatcode']) && $country == 'IT')) {
        $errors->add('validation', __('You should fill either Fiscal code or Vat code field', 'fattura24'));
    } elseif ($country == 'IT' && $cfRequired && empty($_POST['billing_fiscalcode'])) {
        $errors->add('validation', __('Codice fiscale obbligatorio', 'fattura24'));
    } elseif ($country == 'IT' && !empty($_POST['billing_fiscalcode']) && !$cValid) {
        $errors->add('validation', __('Fiscal code should be long 11 or 16 characters', 'fattura24'));
    } elseif ($piRequired && empty($_POST['billing_vatcode'])) {
        $errors->add('validation', __('Vat code required', 'fattura24'));
    } elseif (!empty($_POST['billing_vatcode']) && strpos(strtoupper($_POST['billing_vatcode']), 'IT') !== false) {
        $errors->add('validation', __('Vat code should not contain country IT', 'fattura24'));
    } elseif (!empty($_POST['billing_vatcode']) && $country == 'IT' && !$pILen) {
        $errors->add('validation', __('Vat code should be long 11 characters', 'fattura24'));
    }
}

function fatt_24_add_fieldset_to_my_account()
{
    if (fatt_24_customer_use_cf() || fatt_24_customer_use_vat()) {
        $user_id = get_current_user_id();
        $user = get_userdata($user_id); ?>
    
            <fieldset>
                <legend>
                    <?php _e('Dati fiscali', 'fattura24') ?> <!-- edited label Davide Iandoli 29.01.2019 -->
                </legend>
                <?php if (fatt_24_customer_use_cf()) { ?>
                    <p class="woocommerce-FormRow woocommerce-FormRow--wide form-row form-row-wide"> <!-- edited style -->
                        <label for="billing_fiscalcode">
                            <?php __('Codice Fiscale', 'fattura24') ?>
                        </label>
    
                        <input type="text" class="woocommerce-Input input-text"
                               name="billing_fiscalcode" id="billing_fiscalcode"
                               value="<?php echo esc_attr(strtoupper(fatt_24_user_fiscalcode($user_id))) ?>" />
    
                        <span class="description">
                            <?php _e('inserisci un codice fiscale valido', 'fattura24') ?> <!-- edited label Davide Iandoli 29.01.2019 -->
                        </span> 
                    </p>
                <?php } ?>
         
                <?php if (fatt_24_customer_use_vat()) { ?>
                    <p class="woocommerce-FormRow woocommerce-FormRow--wide form-row form-row-wide">
                        <label for="billing_vatcode">
                            <?php _e('Partita Iva', 'fattura24')?>
                        </label>
    
                        <input type="text" class="woocommerce-Input input-text"
                               name="billing_vatcode" id="billing_vatcode"
                               value="<?php echo esc_attr(fatt_24_user_vatcode($user_id)) ?>" />
    
                        <span class="description">
                            <?php _e('inserisci una Partita Iva valida', 'fattura24') ?> <!-- edited label Davide Iandoli 29.01.2019 -->
                        </span> 
                    </p>
                  <?php } ?> 
                    <p class="woocommerce-FormRow woocommerce-FormRow--wide form-row form-row-wide">
                        <label for="billing_recipientcode">
                            <?php _e('Codice Destinatario', 'fattura24')?>
                        </label>
    
                        <input type="text" class="woocommerce-Input input-text"
                               name="billing_recipientcode" id="billing_recipientcode"
                               maxlength="7"
                               value="<?php echo esc_attr(fatt_24_user_recipientcode($user_id)) ?>" />
    
                        <span class="description">
                            <?php _e('inserire un codice destinatario valido') ?> 
                            <p class="woocommerce-FormRow woocommerce-FormRow--wide form-row form-row-wide">
                        
                        <label for="billing_pec">
                            <?php _e('Indirizzo PEC', 'fattura24')?>
                        </label>
            
                        <input type="email" class="woocommerce-Input input-text"
                               name="billing_pecaddress" id="billing_pecaddress"
                               value="<?php echo esc_attr(fatt_24_user_pecaddress($user_id)) ?>" />
               
                        <span class="description">
                            <?php _e('inserire un indirizzo PEC valido', 'fattura24'); ?> 
            </fieldset> 
    
        <div class="clear"></div>
    <?php
    }
}

function fatt_24_customer_meta_fields($fields)
{
    if (fatt_24_customer_use_cf() || fatt_24_customer_use_vat()) {
        if (!isset($fields['fatt_24'])) {
            $fields['fatt_24'] = array(
                'title' => __('Invoicing required fields', 'fattura24'),
                'fields' => array()
            );
        }
        if (fatt_24_customer_use_cf()) {
            $fields['fatt_24']['fields']['billing_fiscalcode'] = array(
                'label' => __('Fiscal Code', 'fattura24'),
                'description' => __('a valid Fiscal Code', 'fattura24'),
            );
        }
        if (fatt_24_customer_use_vat()) {
            $fields['fatt_24']['fields']['billing_vatcode'] = array(
                'label' => __('Vat code', 'fattura24'),
                'description' => __('a valid Vat Code', 'fattura24'),
            );
        }
    }

    if (fatt_24_customer_use_recipientcode()) {
        $fields['fatt_24']['fields']['billing_recipientcode'] = array(
            'label' => __('Recipient Code', 'fattura24'),
            'description' => __('a valid Recipient Code', 'fattura24'),
        );

        $fields['fatt_24']['fields']['billing_pecaddress'] = array(
            'label' => __('Indirizzo PEC', 'fattura24'),
            'description' => __('a valid PEC address', 'fattura24'),
        );
    }
    return $fields;
}

function fatt_24_created_customer($customer_id)
{
    $f24billingFields = ['billing_fiscalcode', 'billing_vatcode', 'billing_pecaddress', 'billing_recipientcode'];

    foreach ($f24billingFields as $field) {
        if (isset($_POST[$field])) {
            update_user_meta($customer_id, $field, sanitize_text_field($_POST[$field]));
        }
    }
}

function fatt_24_save_account_details($user_id)
{
    if (fatt_24_customer_use_cf()) {
        update_user_meta($user_id, 'billing_fiscalcode', htmlentities(sanitize_text_field(strtoupper($_POST['billing_fiscalcode']))));
    }

    if (fatt_24_customer_use_vat()) {
        update_user_meta($user_id, 'billing_vatcode', htmlentities(sanitize_text_field($_POST['billing_vatcode'])));
    }

    if (fatt_24_customer_use_recipientcode()) {
        update_user_meta($user_id, 'billing_recipientcode', htmlentities(sanitize_text_field($_POST['billing_recipientcode'])));
        update_user_meta($user_id, 'billing_pecaddress', htmlentities(sanitize_text_field($_POST['billing_pecaddress'])));
    }
}

function fatt_24_update_order_meta($order_id)
{
    $f24billingFields = ['billing_fiscalcode', 'billing_vatcode', 'billing_pecaddress', 'billing_recipientcode'];

    foreach ($f24billingFields as $field) {
        if (!empty(sanitize_text_field($_POST[$field]))) {
            update_post_meta($order_id, "_$field", sanitize_text_field($_POST[$field]));
        }
    }
}

function fatt_24_add_fields_to_admin_order($order)
{
    /**
     * Added link to check cf and vat code on AdE services
     * Davide Iandoli 15.11.2021
     */
    $cfValue = strtoupper(fatt_24_order_c_fis($order));
    $checkCf = !empty($cfValue) ?
    '<a href="https://telematici.agenziaentrate.gov.it/VerificaCF/VerificaCf.do?cf=' . $cfValue . '"target="_blank">' . __('(Check fiscal code)', 'fattura24') . '</a>' :
    '' ;
    $cfLabel = '<p><strong>' . __('Fiscal Code', 'fattura24') . ':</strong>' . $cfValue . ' ' . $checkCf . '</p>';

    $piValue = strtoupper(fatt_24_order_p_iva($order));
    $checkPiva = !empty($piValue) ?
   '<a href="https://telematici.agenziaentrate.gov.it/VerificaPIVA/VerificaPiva.do?piva=' . $piValue . '"target="_blank">'. __('(Check vat code)', 'fattura24') . '</a>' :
   '';
    $piLabel = '<p><strong>' . __('Vat Code', 'fattura24') . ':</strong>' . $piValue . ' ' . $checkPiva . '</p>'; ?>
    <div class="address">
		<?php
        if (fatt_24_customer_use_cf()) {
            echo $cfLabel;
        }
    if (fatt_24_customer_use_vat()) {
        echo $piLabel;
    }
    if (fatt_24_customer_use_recipientcode()) {
        echo '<p><strong>' . __('Recipient Code', 'fattura24') . ':</strong>' . fatt_24_order_recipientcode($order).'</p>';
        echo '<p><strong>' . __('PEC address', 'fattura24') . ':</strong>'  .fatt_24_order_pec_address($order). '</p>';
    } ?>
	</div>
    <div class="edit_address">
    <?php
        // con questo codice li rendo editabili nell'ordine lato admin usando il metodo woocommerce_wp_input
        if (fatt_24_customer_use_cf()) {
            woocommerce_wp_text_input(array( 'id' => '_billing_fiscalcode', 'label' =>__('Fiscal Code', 'fattura24'), 'wrapper_class' => '_billing_company_field' ));
        }
    if (fatt_24_customer_use_vat()) {
        woocommerce_wp_text_input(array( 'id' => '_billing_vatcode', 'label' => __('Vat Code', 'fattura24'), 'wrapper_class' => '_billing_company_field' ));
    }
    if (fatt_24_customer_use_recipientcode()) {
        woocommerce_wp_text_input(array( 'id' => '_billing_recipientcode', 'label' => __('Recipient Code', 'fattura24'), 'wrapper_class' => '_billing_company_field' ));
        woocommerce_wp_text_input(array( 'id' => '_billing_pecaddress', 'label' => __('PEC Address', 'fattura24'), 'wrapper_class' => '_billing_company_field' ));
    } ?>
    </div>
    <?php
}

function fatt_24_process_order_meta($post_id, $post)
{
    $f24billingFields = ['billing_fiscalcode', 'billing_vatcode', 'billing_pecaddress', 'billing_recipientcode'];
    foreach ($f24billingFields as $field) {
        update_post_meta($post_id, "_$field", wc_clean(sanitize_text_field($_POST[ "_$field" ])));
    }
}

/**
 * Metodo con cui mostro o nascondo i campi aggiuntivi in base ad alcune opzioni
 * oppure in base al paese di fatturazione selezionato
 * edit 03.03.2022 : ora richiamo js esterni a cui passo i dati necessari
 */
function fatt_24_manage_checkout()
{
    $title = __('Fill in at least one of these two fields', 'fattura24');
    $checkLabel = __('click here to verify', 'fattura24');
    $f24CheckoutJs = ['f24_checkbox_change', 'f24_select_change'];

    $data = [
             'alwaysCreateFE' => get_option('fatt-24-inv-create') == '2' && fatt_24_get_flag(FATT_24_INV_DISABLE_RECEIPTS),
             'showCheckbox' => fatt_24_show_billing_checkbox(),
             'cfRequired' => fatt_24_CF_flag() == 1,
             'pIRequired' => fatt_24_get_flag(FATT_24_ABK_VATCODE_REQ) == 1,
             'error_message' => $title,
             'checkCF' => '<a style="float:right;" href="https://telematici.agenziaentrate.gov.it/VerificaCF/Scegli.do?parameter=verificaCf" target="_blank">' . $checkLabel . '</a>' ,
             'checkPI' => '<a style="float:right;" href="https://telematici.agenziaentrate.gov.it/VerificaPIVA/Scegli.do?parameter=verificaPiva" target="_blank">'. $checkLabel . '</a>'
            ];

    foreach ($f24CheckoutJs as $addJs) {
        wp_enqueue_script($addJs, fatt_24_url('/js/checkout/'. $addJs . '.js'), array('jquery'));
        wp_localize_script($addJs, 'f24_checkout_vars', $data);
    }
}
