<?php

/**
 * Questo file è parte del plugin WooCommerce v3.x di Fattura24
 * Autore: Fattura24.com <info@fattura24.com>
 *
 * metodi utilizzati da hooks.php
 * per l'aggiunta la gestione e la visualizzazione
 * di colonne aggiuntive nella lista degli ordini lato admin
 *
 */

namespace fattura24;

if (!defined('ABSPATH')) {
    exit;
}

function fatt_24_CF_flag()
{
    $result = get_option("fatt-24-abk-fiscode-req") == 1 ? true : false;
    return $result;
}

function fatt_24_woo_checkout_hooked()
{
    return has_action('woocommerce_checkout_fields') == true;
}

function fatt_24_handle_checkout_hooks()
{
    $resultHook = [array('hook' => 'woocommerce_checkout_fields', 'action' => 'fatt_24_billing_checkout_fields'),
                   array('hook' => 'woocommerce_edit_account_form', 'action' => 'fatt_24_add_fieldset_to_my_account')];

    return $resultHook;
}

function fatt_24_array_key_last($array)
{
    $key = null;
    if (is_array($array)) {
        end($array);
        $key = key($array);
    }
    return $key;
}

// con questi metodi gestisco l'aggiornamento dei dati fiscali nell'account utente e ne consento il salvataggio
function fatt_24_user_vatcode($user_id)
{
    if (empty($user_id)) {
        return "";
    } else {
        return get_the_author_meta('billing_vatcode', $user_id);
    }
}

function fatt_24_user_fiscalcode($user_id)
{
    if (empty($user_id)) {
        return "";
    }
    return get_the_author_meta('billing_fiscalcode', $user_id);
}

function fatt_24_user_recipientcode($user_id)
{
    if (empty($user_id)) {
        return "";
    }
    return get_the_author_meta('billing_recipientcode', $user_id);
}

function fatt_24_user_pecaddress($user_id)
{
    if (empty($user_id)) {
        return "";
    }
    return get_the_author_meta('billing_pecaddress', $user_id);
}

function fatt_24_checkout_order_processed($order_id)
{
    static $updated;
    $updated = fatt_24_get_installation_log('triggered update actions');

    if (!$updated) {
       fatt_24_update_actions();
       fatt_24_insert_installation_log('triggered update actions');
       $updated = true;
    }
    $order = new \WC_Order($order_id);
    $isZeroOrderEnabled = (int) get_option('fatt-24-ord-zero-tot-enable') == 1;
    $isOrderTotalZero = (float) $order->get_total() == 0.00;
    $isOrderEnabled = (int) get_option(FATT_24_ORD_CREATE) == 1;

    if (($isOrderEnabled && !$isOrderTotalZero)  || ($isOrderEnabled && $isZeroOrderEnabled)) {
        fatt_24_process_order($order_id);
        //fatt_24_download_PDF_order($order_id);
    }
}

function fatt_24_manage_admin_order($order_id)
{
    static $updated;
    $updated = fatt_24_get_installation_log('triggered update actions');

    if (!$updated) {
       fatt_24_update_actions();
       fatt_24_insert_installation_log('triggered update actions');
       $updated = true;
    }

    /**
     * Con questo blocco di codice punto a evitare che vengano creati ordini
     * in uno degli stati elencati nell'array. Aggiunta l'opzione 'Mai'
     *
     * Fix del 21.12.2020: aggiunto controllo sul tipo di post
     * non voglio interferire sull'aggiunta di prodotti, coupon etc.
     */
    $disabledOrder = get_option(FATT_24_ORD_CREATE) == 0;
    $orderStatusChosen = get_option(FATT_24_ORD_CREATE) == 2;
    $selectedStatusOption = get_option(FATT_24_ORD_STATUS_SELECT);
    $postTypesArray = ['shop_order', 'shop_subscription'];
    $postType = get_post_type($order_id);
    $postStatus = get_post_status($order_id);
    $postStatusArray = ['wc-cancelled', 'wcm-cancelled', 'wc-pending-cancel', 'trash'];
    $postTypeAllowed = in_array($postType, $postTypesArray) && !in_array($postStatus, $postStatusArray);

    if ($disabledOrder || !$postTypeAllowed) {
        return;
    }

    $order = new \WC_Order($order_id);
    $order_data = $order->get_data();
    $order_status = $order_data['status'];
    $isOrderStatusSelected = ($selectedStatusOption == 0 && $order_status == 'completed') || ($selectedStatusOption == 1 && $order_status == 'processing');

    if ($orderStatusChosen && !$isOrderStatusSelected) {
        return;
    }

    if (!did_action('woocommerce_checkout_order_processed') && $postTypeAllowed && fatt_24_validate_order($order_id)) {
        fatt_24_process_order($order_id);
        //fatt_24_download_PDF_order($order_id);
    }
}

//cfr.: https://stackoverflow.com/questions/54347823/get-last-old-order-status-before-updated-status-in-woocommerce
function fatt_24_order_status_changed($order_id, $status_from, $status_to, $order)
{
    if ($order->get_meta('_old_status')) {
        update_post_meta($order_id, '_old_status', $status_from);
    } else {
        update_post_meta($order_id, '_old_status', 'pending');
    }
}

function fatt_24_trigger_invoice_on_order_status($order_id)
{
    $order = new \WC_Order($order_id);
    $order_old_status = get_post_meta($order_id, '_old_status', true);
    $createBothDocs = get_option(FATT_24_ORD_CREATE) == '2';

    /**
    * Codice modificato il 18.5.2021
    * Crea la fattura o ricevuta con totale a zero se è contrassegnata l'opzione
    * oppure se il totale ordine NON è zero
    */

    $isZeroInvoiceEnabled = (int) get_option('fatt-24-inv-zero-tot-enable') == 1;
    $isZeroOrderEnabled = (int) get_option('fatt-24-ord-zero-tot-enable') == 1;
    $isOrderTotalZero = (float) $order->get_total() == 0.00;


    if ($order_old_status !== 'processing' || $order_old_status !== 'completed') {
        if ($createBothDocs) {
            if ($isZeroOrderEnabled || !$isOrderTotalZero) {
                fatt_24_process_order($order_id);
                sleep(15); // intervallo di 15" prima di chiamare nuovamente
            }

            if ($isZeroInvoiceEnabled || !$isOrderTotalZero) {
                fatt_24_process_fattura($order_id);
            }
        } else {
            if ($isZeroInvoiceEnabled || !$isOrderTotalZero) {
                fatt_24_process_fattura($order_id);
            }
        }
    }
}

/**
 * Mostra i link nella sezione 'il mio account'
 *
 */
function fatt_24_display_PDF($actions, $order)
{
    $invType = fatt_24_get_invoice_doctype();
    $order_id = $order->get_id();
    $order_status = fatt_24_get_order_status($order_id);
    $orderLink = isset(fatt_24_get_pdf_link($order_status, 'C')['pdfPath'])
                 ? fatt_24_get_pdf_link($order_status, 'C')['pdfPath'] : '';
    $invLink = isset(fatt_24_get_pdf_link($order_status, $invType)['pdfPath'])
                 ? fatt_24_get_pdf_link($order_status, $invType)['pdfPath'] : '' ;
    $orderPDFUrl = fatt_24_order_PDF_url($orderLink);
    $invPDFUrl = fatt_24_order_PDF_url($invLink);
    $viewPdfOrder = isset($orderLink) && file_exists($orderLink);
    $viewPdfInvoice = isset($invLink) && file_exists($invLink);

    if ($viewPdfOrder) {
        $actions['orderPdfView'] = array(
            'url' => $orderPDFUrl,
            'name' => __('PDF Order', 'fattura24'),
        );
    }

    if ($viewPdfInvoice) {
        $actions['invoicePdfView'] = array(
            'url' => $invPDFUrl,
            'name' => __('PDF Invoice', 'fattura24'),
        );
    }

    return $actions;
}

function fatt_24_action_after_account_orders_js()
{
    $addJs = 'f24_my_account_actions';
    wp_enqueue_script($addJs, fatt_24_url('/js/myaccount/'. $addJs . '.js'), array());
}
