<?php
/**
 * Questo file è parte del plugin WooCommerce v3.x di Fattura24
 * Autore: Fattura24.com <info@fattura24.com>
 *
 * Metodi comuni a tutte le schermate di impostazioni:
 * elenco delle schermate, barra di navigazione, rating e infobox
 */

namespace fattura24;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Metodo con cui costruisco l'elenco delle tabs nella schermata di admin
 */
function fatt_24_settings_tabs()
{
    $fatt24Menus = [
        array('slug' => 'fatt-24-settings', 'label' => __('General Settings', 'fattura24')),
        array('slug' => 'fatt-24-tax', 'label' => __('Tax configuration', 'fattura24')),
        array('slug' => 'fatt-24-api-info', 'label' => __('API calls', 'fattura24')),
        array('slug' => 'fatt-24-support', 'label' => __('Support', 'fattura24')),
        array('slug' => 'fatt-24-videos', 'label' => fatt_24_video_icon() . __(' Video guides', 'fattura24')),
        array('slug' => 'fatt-24-app', 'label' => __('Mobile app', 'fattura24')),
        array('slug' => 'fatt-24-warning', 'label' => fatt_24_notice(). __(' Warning', 'fattura24')),
    ];
    return $fatt24Menus;
}

/**
 * ottengo la pagina attuale dalla url
 */
function fatt_24_get_page()
{
    $url = $_SERVER['REQUEST_URI'];
    $page = substr($url, strpos($url, 'page=') + 5);
    return $page;
}

/**
 * Barra di navigazione
 */
function fatt_24_build_nav_bar()
{
    $nav = '<nav class="nav-tab-wrapper woo-nav-tab-wrapper">';
    $tabs = fatt_24_settings_tabs();
    $activeTab = fatt_24_get_page();
       
    foreach ($tabs as $tab) {
        if ($tab['slug'] == $activeTab) {
            $nav .= '<a href="?page=' . $tab['slug'] . '"class="nav-tab nav-tab-active">' . $tab['label'] . '</a>';
        } else {
            $nav .= '<a href="?page=' . $tab['slug'] . '"class="nav-tab">' . $tab['label'] . '</a>';
        }
    }
    $nav .= '</nav>';
    return $nav;
}

function fatt_24_headers_style()
{
    return array('open' => '<div style="border-bottom:1px solid; padding-bottom: 10px; padding-top: 20px;">',
                 'close' => '</div>');
}

/**
 * Con questo metodo controllo se è già stata inserita una valutazione
 * i rating vengono registrati in wp_option (opzione fatt-24-woo-rating)
 * e confrontati con la data di oggi.  
 * 
 * @return $hidden => se la data dell'ultimo rating è maggiore di 6 mesi
 * 
 */
function fatt_24_getRating()
{
    $f24WooRatingValue = get_option('fatt-24-woo-rating');
    $hidden = false;
    if (!isset($f24WooRatingValue) || $f24WooRatingValue == '0') {
         return $hidden;
    }

    $today = \DateTime::createFromFormat('d-m-Y', date('d-m-Y'));
    $lastReview = json_decode($f24WooRatingValue);

    if (is_object($lastReview)) {
        $lastReviewDateString = $lastReview->lastUpdate;
        $lastReviewDateToTime = strtotime($lastReviewDateString);
        $lastReviewDate = \DateTime::createFromFormat('d-m-Y', date('d-m-Y', $lastReviewDateToTime));
        $comparison = $lastReviewDate->diff($today);
        $different_month = $comparison->m;
        $hidden = $different_month < 6;
    }
    
    return $hidden;
}

/**
 *  Con questo metodo costruisco un array di 5 righe
 *  per 5 stelle; per ogni singola stella cambia solo il background
 *  per ogni singola riga (dalla 2a) viene applicato un background diverso
 */
function fatt_24_build_star_rows()
{
    /**
     *  Array di colori background: il primo codice
     *  corrisponde al grigio di default, gli altri:
     *  1 => rosso, una stella sola
     *  2 => arancione, due stelle
     *  3 => giallo, tre stelle
     *  4 => verde chiaro, 4 stelle
     *  5 => verde smeraldo, 5 stelle
     */
    $backgrounds = ['#fd3722;', '#fd8621;', '#fdce00;', '#72cf0f;', '#00b77b;', '#dbdbe3;'];
    $backgroundProp = 'background-color:';
    $styleStar = 'color: #fff; margin: 2px; padding: 2px;';
    $singleStar = array();
    $starsRow = array();
    $rowCounter = 1;
    while ($rowCounter <= 5) {
        for ($i = 0; $i<5; $i++) {
            switch ($rowCounter) {
                case 1:
                    $background = $i < $rowCounter ? $backgrounds[0] : end($backgrounds);
                    $singleStar[$i] = "<span style='$backgroundProp $background $styleStar' class='dashicons dashicons-star-filled'></span>";
                break;
                case 2:
                    $background = $i < $rowCounter ? $backgrounds[1] : end($backgrounds);
                    $singleStar[$i] = "<span style='$backgroundProp $background $styleStar' class='dashicons dashicons-star-filled'></span>";
                break;
                case 3:
                    $background = $i < $rowCounter ? $backgrounds[2] : end($backgrounds);
                    $singleStar[$i] = "<span style='$backgroundProp $background $styleStar' class='dashicons dashicons-star-filled'></span>";
                break;
                case 4:
                    $background = $i < $rowCounter ? $backgrounds[3] : end($backgrounds);
                    $singleStar[$i] = "<span style='$backgroundProp $background $styleStar' class='dashicons dashicons-star-filled'></span>";
                break;
                case 5:
                    $background = $i < $rowCounter ? $backgrounds[4] : end($backgrounds);
                    $singleStar[$i] = "<span style='$backgroundProp $background $styleStar' class='dashicons dashicons-star-filled'></span>";
                break;
                default:
                break;
            }
        }
        $starsRow[] = $singleStar;
        $rowCounter++;
    }
    return $starsRow;
}


// link alla documentazione F24
function fatt_24_doc_links()
{
    $links = [['link' => esc_url(admin_url('options-general.php?page=fatt-24-support')),
               'label' => __('Contact our tech service', 'fattura24')],
              ['link' => 'https://www.fattura24.com/condizioni-di-contratto',
               'label' => __('Fattura24 general terms and conditions of contract', 'fattura24')],
              ['link' => 'https://www.fattura24.com/regolamento-ecommerce',
               'label' => __('Supplementary F24 regulation for WooCommerce plugin', 'fattura24')],
              ['link' => 'https://www.fattura24.com/informativa-privacy',
               'label' => __('Privacy policy', 'fattura24')]
            ];
    return $links;
}

/**
 * Info box visibile in tutte le tab tranne l'ultima
 * Nella schermata di impostazioni generali posso visualizzare il box con il rating
 * collegato all'opzione F24_WOO_RATING se non è mai stato fatto click sull'opzione
 * oppure se l'ultima volta il click è stato fatto almeno 6 mesi fa
 */ 
function fatt_24_infobox($page = null)
{
    $hidden = fatt_24_getRating(); 
    $sendReview = '';
    $pStyle = 'margin:1px;';
    $links = fatt_24_doc_links();
    // voglio visualizzare i punteggi a partire dal più alto
    $buildedStarsRows = array_reverse(fatt_24_build_star_rows());
    $html = "<div style='background: #ffc; border: 1px solid #ccd0d4; margin: 10px; padding: 15px;'>";
    
    if (!$hidden && $page == 'fatt-24-settings') {
        $html .= "<div style='margin-bottom: 30px;' id='reviewContainer'>";
        $html .= "<h4>" . __('How would you rate Fattura24?', 'fattura24') . "</h4>";
        $html .= __('Write down a review and help us to improve.
			  All the reviews, either positive or negative, will be displayed immediately.', 'fattura24');
    
        // array delle righe
        $radioValue = 5;
        foreach ($buildedStarsRows as $key => $rowStars) {
            $html .= $key == 0? "<p style='margin-bottom: 1px;'>" : "<p style='$pStyle'>";
            $html .= "<div style='margin-bottom: -10px; cursor:pointer;' id='reviewBox$radioValue'>";
            $html .= "<input id='radioReview' style='cursor:pointer; background-color: #fff; margin: 5px;' type='radio' name='review' value='$radioValue' />";
            
            //array delle singole stelle
            foreach ($rowStars as $star) {
                $html .= $star;
            }
            $html .= "</div>";
            $html .= "</p>";
            $radioValue--;
        }
        $html .= "</div>";
    }
    
    $html .= "</p>";
    $html .= "<div style='margin-top: -10px;' id='linkContainer'>";
    $html .= "<h4>" . __('Do you need help ?', 'fattura24') . "</h4>";
    
    // array dei link
    foreach ($links as $link) {
        //var_dump($link);
        if (strpos($link['link'], 'fatt-24-support') !== false) {
            $html .= "<p><a style='text-decoration:none;' href=" . $link['link'] . ">";
        } else {
            $html .= "<p><a style='text-decoration:none;' href=" . $link['link'] . " target='_blank'>";
        }
        $html .= $link['label'] . "</a></p>";
    }

    $html .= "</div>";
    $html .= "<a style='text-align:center; display:block;' class='button button-primary' href='https://www.app.fattura24.com/v3' target='_blank'>";
    $html .= __('Log in Fattura24', 'fattura24');
    $html .= "</a>";
    $html .= "</div>";
        
    return $html;
}

/**
 * nuove opzioni del menu crea documento fiscale
 * Davide Iandoli 14.10.2021
 */
function fatt_24_getInvoiceOptionsNew()
{
    return array( '0' => __('Disabled', 'fattura24'),
                  '1' => __('Non-electronic Invoice', 'fattura24'),
                  '2' => __('Electronic Invoice', 'fattura24'),
                  '3' => __('Electronic Invoice (if IT customer else Receipt)', 'fattura24'),
                  '4' => __('Electronic Invoice (if IT customer else Invoice)', 'fattura24'),
                  '6' => __('Electronic Invoice (if EU customer else Receipt)', 'fattura24'),
                  '7' => __('Electronic Invoice (if EU customer else Invoice)', 'fattura24'),
                  '5' => __('Non-fiscal Receipt', 'fattura24')
            );
}